﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
namespace DHTMLX.Export.PDF.Scheduler
{
    public enum ColorProfile
    {
        Color,
        FullColor,
        Gray,
        BW,
        Custom
    }



    public class XMLParser
    {
        private string xml;
	    private XmlNode root;
	    private string mode;
	    private string todayLabel;
	    private string[,] rows;
        private ColorProfile profile = ColorProfile.Color;
	    private string header = "false";
	    private string footer = "false";
	    private List<SchedulerEvent> multiday = new List<SchedulerEvent>();
	    private List<SchedulerEvent> events = new List<SchedulerEvent>();
	    private string[] cols = null;


        public ColorProfile StringToColorProfile(string profile)
        {
            if (profile == null)
                return ColorProfile.Color;
            switch (profile.ToLower())
            {
                case "gray":
                    {
                        return ColorProfile.Gray;

                    }
                case "full_color":
                    {
                        return ColorProfile.FullColor;
                    }
                case "color":
                    {
                        return ColorProfile.Color;
                    }
                case "bw":
                    {
                        return ColorProfile.BW;
                    }
                case "custom":{
                    return ColorProfile.Custom;
                }
                default:
                    return ColorProfile.Color;

            }
        }

        public static List<XmlNode> GetElementsByTagName(XmlNode parent, string name)
        {
            var list = new List<XmlNode>();
            foreach (XmlNode node in parent.ChildNodes)
            {
                if (node.LocalName == name)
                {
                    list.Add(node);
                }
                else
                {
                    list.AddRange(GetElementsByTagName(node, name));
                }
            }
            return list;
        }
	    public void SetXML(string xml){
		    this.xml = xml;

            XmlDocument dom = new XmlDocument(); 
           
            dom.LoadXml(xml);

            root = dom.DocumentElement;

            XmlNode scale = null;
            foreach (XmlNode nod in root.ChildNodes)
            {
                if (nod.LocalName == "scale")
                {
                    scale = nod;
                    break;
                }
            }

            if (root.Attributes["profile"] != null)
            {
                this.profile = StringToColorProfile(root.Attributes["profile"].Value);
            }
            else if (scale.Attributes["profile"] != null)
            {
                this.profile = StringToColorProfile(scale.Attributes["profile"].Value);
            }

            if (root.Attributes["header"] != null)
            {
                this.header = (root.Attributes["header"].Value);
            }
            else if (scale.Attributes["header"] != null)
            {
                this.header = (scale.Attributes["header"].Value);
            }

            if (root.Attributes["footer"] != null)
            {
                this.footer = (root.Attributes["footer"].Value);
            }
            else if (scale.Attributes["footer"] != null)
            {
                this.footer = (scale.Attributes["footer"].Value);
            }

		    this.mode = scale.Attributes["mode"] != null ? scale.Attributes["mode"].Value : "";
            this.todayLabel = scale.Attributes["today"] != null ? scale.Attributes["today"].Value : "";

		    this.EventsParsing();
	    }

	    public String[] MonthColsParsing() {

            var n1 = GetElementsByTagName(root, "column"); 
		    if ((n1 != null)&&(n1.Count > 0)) {
                this.cols = new String[n1.Count];
                for (int i = 0; i < n1.Count; i++)
                {
				    var col = n1[i];
				    this.cols[i] = col.FirstChild.Value;
			    }
		    }
		    return this.cols;
	    }
	
	    public String[,] MonthRowsParsing() {
            var n1 = GetElementsByTagName(root, "row");
		    if ((n1 != null)&&(n1.Count > 0)) {
			    this.rows = new string[n1.Count,7];
			    for (int i = 0; i < n1.Count; i++) {
				    var row = n1[i];
				    String week = row.FirstChild.Value;
                    var days = week.Split(new string[1]{"|"}, StringSplitOptions.None );
                    for (var j = 0; j < days.Length; j++)
                        this.rows[i, j] = days[j];
			    }
		    }
		    return this.rows;
	    }

	    public void EventsParsing() {
		    var n1 = GetElementsByTagName(root, "event");
		    if ((n1 != null)&&(n1.Count > 0)) {
                for (int i = 0; i < n1.Count; i++)
                {
				    var ev = n1[i];
				    SchedulerEvent oEv = new SchedulerEvent();
				    oEv.Parse(ev);
                    if ((oEv.getType() == "event_line") && (this.mode != "month") && (this.mode != "timeline") && (this.mode != "treetimeline"))
                    {
					    multiday.Add(oEv);
				    } else {
					    events.Add(oEv);
				    }
			    }
		    }
	    }

	    public SchedulerEvent[] GetEvents() {
            SchedulerEvent[] events_list = new SchedulerEvent[events.Count];
		    for (int i = 0; i < events.Count; i++)
			    events_list[i] = events[i];
		    return events_list;
	    }
	
	    public SchedulerEvent[] GetMultidayEvents() {
            SchedulerEvent[] events_list = new SchedulerEvent[multiday.Count];
            for (int i = 0; i < multiday.Count; i++)
			    events_list[i] = multiday[i];
		    return events_list;
	    }

	    public String[] WeekColsParsing() {
		    if (cols != null)
			    return cols;
            var n1 = GetElementsByTagName(root, "column");
            if ((n1 != null) && (n1.Count > 0))
            {
                cols = new String[n1.Count];
                for (int i = 0; i < n1.Count; i++)
                {
				    var col = n1[i];
				    cols[i] = col.FirstChild.Value;
			    }
		    }
		    return cols;
	    }
	
	    public String[] WeekRowsParsing() {
		    String[] rows = null;
		    var n1 =  GetElementsByTagName(root, "row");
		    if ((n1 != null)&&(n1.Count > 0)) {
                rows = new String[n1.Count];
                for (int i = 0; i < n1.Count; i++)
                {
				    var row = n1[i];
				    rows[i] = row.FirstChild.Value;
			    }
		    }
		    return rows;
	    }
	
	    public SchedulerMonth[] YearParsing() {
		    SchedulerMonth[] monthes = null;
            var n1 = GetElementsByTagName(root, "month");
            if ((n1 != null) && (n1.Count > 0))
            {
                monthes = new SchedulerMonth[n1.Count];
                for (int i = 0; i < n1.Count; i++)
                {
				    monthes[i] = new SchedulerMonth();
				    var mon = n1[i];
				    monthes[i].Parse(mon);
			    }
		    }
		    return monthes;
	    }
	
	    public String[] AgendaColsParsing() {
		    String[] cols = null;
            var n1 = GetElementsByTagName(root, "column");
            if ((n1 != null) && (n1.Count > 0))
            {
                cols = new String[n1.Count];
                for (int i = 0; i < n1.Count; i++)
                {
				    var col = n1[i];
                    cols[i] = col.FirstChild.Value;
			    }
		    }
		    return cols;
	    }
	
	    public String GetMode() {
		    return this.mode;
	    }

	    public String GetTodayLabel() {
		    return this.todayLabel;
	    }
	
	    public ColorProfile GetColorProfile() {
		    return this.profile;
	    }
	
	    public bool GetHeader() {
            bool result = false;
		    if (this.header == "true") {
			    result = true;
		    }
		    return result;
	    }

        public bool GetFooter()
        {
            bool result = false;
		    if (this.footer == "true") {
			    result = true;
		    }
		    return result;
	    }
    }
}
